//
//  Controller.m
//  IntelSensors
//
//  Created by iSlayer on 17/04/07.
//  Copyright 2007 iSlayer & Distorted Vista. All rights reserved.

#import "iStatIntelControlleriStatPro.h"
#import <smc.h>
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <IOKit/IOKitLib.h>

@implementation iStatIntelControlleriStatPro

io_connect_t conn2;
kern_return_t result;
SMCVal_t      val;

- (void) dealloc {
	[availableKeys release];
	[supportedKeys release];
	[keyDisplayNames release];
	[priorities release];

	SMCClose(&conn2);
	smc_close();	
	[super dealloc];
}


- (id)init {
	self = [super init];
	result = SMCOpen(&conn2);
    if (result != kIOReturnSuccess){
		supported = NO;
		return self;
	}
	supported = YES;
	smc_init();
	
	supportedKeys = [[NSMutableArray alloc] init];
	[self setKeys];
	[self findSupportedKeys];
	
	return self;
}

- (BOOL)isSupported {
	return supported;
}

- (void)setKeys {
	availableKeys = [[NSMutableArray alloc] init];
	[availableKeys addObject:@"TC0H"];
	[availableKeys addObject:@"TC0D"];
	[availableKeys addObject:@"TC1D"];
	[availableKeys addObject:@"TCAH"];
	[availableKeys addObject:@"TCBH"];
	[availableKeys addObject:@"TG0P"];
	[availableKeys addObject:@"TA0P"];
	[availableKeys addObject:@"TH0P"];
	[availableKeys addObject:@"TO0P"];
	[availableKeys addObject:@"TH1P"];
	[availableKeys addObject:@"TH2P"];
	[availableKeys addObject:@"TH3P"];
	[availableKeys addObject:@"Th0H"];
	[availableKeys addObject:@"Th1H"];
	[availableKeys addObject:@"TG0H"];
	[availableKeys addObject:@"TG0D"];
	[availableKeys addObject:@"TM0S"];
	[availableKeys addObject:@"TM1S"];
	[availableKeys addObject:@"TM2S"];
	[availableKeys addObject:@"TM3S"];
	[availableKeys addObject:@"Tp1C"];
	[availableKeys addObject:@"Tp0C"];
	[availableKeys addObject:@"TB0T"];
	[availableKeys addObject:@"TN0P"];
	[availableKeys addObject:@"TN1P"];

	keyDisplayNames = [[NSMutableDictionary alloc] init];
	[keyDisplayNames setValue:@"CPU A" forKey:@"TC0H"];
	[keyDisplayNames setValue:@"CPU A" forKey:@"TC0D"];
	[keyDisplayNames setValue:@"CPU B" forKey:@"TC1D"];
	[keyDisplayNames setValue:@"CPU A" forKey:@"TCAH"];
	[keyDisplayNames setValue:@"CPU B" forKey:@"TCBH"];
	[keyDisplayNames setValue:@"GPU" forKey:@"TG0P"];
	[keyDisplayNames setValue:@"Ambient" forKey:@"TA0P"];
	[keyDisplayNames setValue:@"HD Bay 1" forKey:@"TH0P"];
	[keyDisplayNames setValue:@"HD Bay 2" forKey:@"TH1P"];
	[keyDisplayNames setValue:@"HD Bay 3" forKey:@"TH2P"];
	[keyDisplayNames setValue:@"HD Bay 4" forKey:@"TH3P"];
	[keyDisplayNames setValue:@"Optical Drive" forKey:@"TO0P"];
	[keyDisplayNames setValue:@"Heatsink A" forKey:@"Th0H"];
	[keyDisplayNames setValue:@"Heatsink B" forKey:@"Th1H"];
	[keyDisplayNames setValue:@"GPU Diode" forKey:@"TG0D"];
	[keyDisplayNames setValue:@"GPU Heatsink" forKey:@"TG0H"];
	[keyDisplayNames setValue:@"Mem module 1" forKey:@"TM0S"];
	[keyDisplayNames setValue:@"Mem module 2" forKey:@"TM1S"];
	[keyDisplayNames setValue:@"Mem module 3" forKey:@"TM2S"];
	[keyDisplayNames setValue:@"Mem module 4" forKey:@"TM3S"];
	[keyDisplayNames setValue:@"Power supply 2" forKey:@"Tp1C"];
	[keyDisplayNames setValue:@"Power supply 1" forKey:@"Tp0C"];
	[keyDisplayNames setValue:@"Enclosure Bottom" forKey:@"TB0T"];
	[keyDisplayNames setValue:@"Northbridge 1" forKey:@"TN0P"];
	[keyDisplayNames setValue:@"Northbridge 2" forKey:@"TN1P"];

	priorities = [[NSMutableDictionary alloc] init];
	[priorities setValue:[NSNumber numberWithInt:1] forKey:@"TC0H"];
	[priorities setValue:[NSNumber numberWithInt:1] forKey:@"TC0D"];
	[priorities setValue:[NSNumber numberWithInt:1] forKey:@"TC1D"];
	[priorities setValue:[NSNumber numberWithInt:1] forKey:@"TCAH"];
	[priorities setValue:[NSNumber numberWithInt:1] forKey:@"TCBH"];
	[priorities setValue:[NSNumber numberWithInt:2] forKey:@"TG0P"];
	[priorities setValue:[NSNumber numberWithInt:2] forKey:@"TA0P"];
	[priorities setValue:[NSNumber numberWithInt:4] forKey:@"TH0P"];
	[priorities setValue:[NSNumber numberWithInt:4] forKey:@"TH1P"];
	[priorities setValue:[NSNumber numberWithInt:4] forKey:@"TH2P"];
	[priorities setValue:[NSNumber numberWithInt:4] forKey:@"TH3P"];
	[priorities setValue:[NSNumber numberWithInt:4] forKey:@"TO0P"];
	[priorities setValue:[NSNumber numberWithInt:2] forKey:@"Th0H"];
	[priorities setValue:[NSNumber numberWithInt:2] forKey:@"Th1H"];
	[priorities setValue:[NSNumber numberWithInt:2] forKey:@"TG0D"];
	[priorities setValue:[NSNumber numberWithInt:2] forKey:@"TG0H"];
	[priorities setValue:[NSNumber numberWithInt:4] forKey:@"TM0S"];
	[priorities setValue:[NSNumber numberWithInt:4] forKey:@"TM1S"];
	[priorities setValue:[NSNumber numberWithInt:4] forKey:@"TM2S"];
	[priorities setValue:[NSNumber numberWithInt:4] forKey:@"TM3S"];
	[priorities setValue:[NSNumber numberWithInt:4] forKey:@"Tp1C"];
	[priorities setValue:[NSNumber numberWithInt:4] forKey:@"Tp0C"];
	[priorities setValue:[NSNumber numberWithInt:4] forKey:@"TB0T"];
	[priorities setValue:[NSNumber numberWithInt:4] forKey:@"TN0P"];
	[priorities setValue:[NSNumber numberWithInt:4] forKey:@"TN1P"];
}

- (void)findSupportedKeys {
    SMCVal_t      val;
	
	NSEnumerator *keyEnumerator = [availableKeys objectEnumerator];
	NSString *key;
	while(key = [keyEnumerator nextObject]){
		result = SMCReadKey2([key cString], &val,conn2);
		if (result == kIOReturnSuccess){
			if (val.dataSize > 0) {
				if(((val.bytes[0] * 256 + val.bytes[1]) >> 2)/64 <= 0)
					continue;
				
				[supportedKeys addObject:key];
			}
		}
	}
}

- (NSArray *)getFans {
    kern_return_t result;
    SMCVal_t      val;
    UInt32Char_t  key;
    int           totalFans, i;

    result = SMCReadKey("FNum", &val);
    if (result != kIOReturnSuccess)
        return [NSArray array];

   totalFans = _strtoul(val.bytes, val.dataSize, 10); 

	NSMutableArray *fans = [[NSMutableArray alloc] init];
    for (i = 0; i < totalFans; i++)
    {
        sprintf(key, "F%dAc", i); 
        SMCReadKey(key, &val); 
		[fans addObject:[NSString stringWithFormat:@"Fan %i",i]];
    }

	return [fans autorelease];
}


// getFanName - originally from smcFanControl
- (NSString *)getFanName:(int)number {
	UInt32Char_t  key;
	char temp;
	SMCVal_t      val;
	kern_return_t result;
	NSMutableString *desc;
	desc = [[NSMutableString alloc]init];
	sprintf(key, "F%dID", number);
	result = SMCReadKey2(key, &val,conn2);
	int i;
	for (i = 0; i < val.dataSize; i++) {
		if ((int)val.bytes[i ] >32) {
			temp = (unsigned char)val.bytes[i];
			[desc appendFormat:@"%c",temp];
		}
	}
	
	if([desc length] == 0)
		[desc setString:[NSString stringWithFormat:@"Fan %i",number]];
	return [desc autorelease];
}	

- (NSDictionary *)getFanValues {
    SMCVal_t      val;
    UInt32Char_t  key;
    int           totalFans, i;

    result = SMCReadKey("FNum", &val);
    if (result != kIOReturnSuccess)
        return [NSDictionary dictionary];

	totalFans = _strtoul(val.bytes, val.dataSize, 10); 

	NSMutableDictionary *fans = [[NSMutableDictionary alloc] init];
    for (i = 0; i < totalFans; i++) {
        sprintf(key, "F%dAc", i); 
        SMCReadKey(key, &val); 
		[fans setValue:[NSString stringWithFormat:@"%@rpm",[NSNumber numberWithInt:_strtof(val.bytes, val.dataSize, 2)]] forKey:[self getFanName:i]];
	}

	return [fans autorelease];
}

- (NSDictionary *)getTempValues {
	NSMutableArray *values = [[NSMutableArray alloc] init];
	NSMutableArray *priorityOne = [[NSMutableArray alloc] init];
	NSMutableArray *priorityTwo = [[NSMutableArray alloc] init];
	NSMutableArray *priorityThree = [[NSMutableArray alloc] init];
	NSMutableArray *priorityFour = [[NSMutableArray alloc] init];

    SMCVal_t val;
	
	NSEnumerator *keyEnumerator = [supportedKeys objectEnumerator];
	NSString *key;
	while(key = [keyEnumerator nextObject]){
		result = SMCReadKey2([key cString], &val, conn2);
		if (result == kIOReturnSuccess){
			if (val.dataSize > 0) {
				if(((val.bytes[0] * 256 + val.bytes[1]) >> 2)/64 == 0)
					continue;
					
				if([[priorities objectForKey:key] intValue] == 1)
					[priorityOne addObject:[NSArray arrayWithObjects:[keyDisplayNames objectForKey:key], [NSNumber numberWithInt:((val.bytes[0] * 256 + val.bytes[1]) >> 2)/64], nil]];
				
				if([[priorities objectForKey:key] intValue] == 2)
					[priorityTwo addObject:[NSArray arrayWithObjects:[keyDisplayNames objectForKey:key], [NSNumber numberWithInt:((val.bytes[0] * 256 + val.bytes[1]) >> 2)/64], nil]];
				
				if([[priorities objectForKey:key] intValue] == 3)
					[priorityThree addObject:[NSArray arrayWithObjects:[keyDisplayNames objectForKey:key], [NSNumber numberWithInt:((val.bytes[0] * 256 + val.bytes[1]) >> 2)/64], nil]];
				
				if([[priorities objectForKey:key] intValue] == 4)
					[priorityFour addObject:[NSArray arrayWithObjects:[keyDisplayNames objectForKey:key], [NSNumber numberWithInt:((val.bytes[0] * 256 + val.bytes[1]) >> 2)/64], nil]];
			}	
		}
	}

	[priorityOne sortUsingSelector:@selector(compare:)];
	[priorityTwo sortUsingSelector:@selector(compare:)];
	[priorityThree sortUsingSelector:@selector(compare:)];
	[priorityFour sortUsingSelector:@selector(compare:)];
	
	[values addObject:priorityOne];
	[values addObject:priorityTwo];
	[values addObject:priorityThree];
	[values addObject:priorityFour];

	[priorityOne release];
	[priorityTwo release];
	[priorityThree release];
	[priorityFour release];

	return [values autorelease];
}

@end
